#!/bin/bash

####Changelog
###Init script ----------2022/06/22
###Relseased   ----------
###Optimize iot_reload ---- 2023/03/18
###
####xiaoye, xiaoye@dragino.cc


###Initialization parameters

HOST1=1.1.1.1
HOST2=8.8.8.8
IOT_TYPE=$(uci -q get gateway.general.server_type)
last_reload_time="0"
internet_offline=0
last_iotoffline_check=0
####Check the internet
check_internet()
{
checkinternet=$(fping $HOST1 | grep alive -c)
if [[ $checkinternet = 1 ]]; then
	internetstatus=1
	echo 1 > /var/iot/internet
else
	checkinternet=$(fping $HOST2 | grep alive -c)
	if [[ $checkinternet = 1 ]]; then
	    internetstatus=1
		echo 1 > /var/iot/internet
	else
		internetstatus=0
		echo 0 > /var/iot/internet
	fi
fi

default_route=$(ip route | grep default -c)
if [[ $default_route = 1 ]]; then
	logger -i -t "iot_keepa_live" "Internet Access: OK: via $(ip route | grep default | awk '{print "Dev:"$5,"Gateway:" $3}') "
elif [[ $default_route = 2 ]]; then
	logger -i -t "iot_keep_alive" "Internet Access: OK: via $(ip route | grep default | awk 'NR==1{print "Dev:" $5,"Gateway:" $3}') or $(ip route | grep default | awk 'NR==2{print "Dev:"$5,"Gateway:" $3}') "
else
	logger -i -t "iot_keep_alive" "Internet Access: Fail, no any routing table"
fi

}

## check the wlan0
check_wlan_status()
{
        sta_status=$(uci get wireless.sta_0.disabled)
        ncm_status=$(nmcli d show wlan0 | grep STATE | awk '{print $3}' | sed 's/.$//' | sed 's/.//')

}


###Check the iot

check_fwd()
{
	checkfwd=$(pgrep fwd)
        if [[ -n $checkfwd ]]; then
            process_status=1
			iot_status=$(sqlite3 /tmp/lgwdb.sqlite "select * from gwdb where key='/service/lorawan/primary_server/network'" | awk -F'|' '{print $2}')
			if [[ $iot_status = "online" ]]; then
				iot_online=1
				echo "online" > /var/iot/status
			else
				iot_online=0
                echo "offline" > /var/iot/status
			fi
        else
			process_status=0
        fi
}

check_station()
{
	checkstation=$(pgrep station)
                if [[ -n $checkstation ]]; then
                        process_status=1
						iot_status=`tail /var/log/station.log | grep -e "HTTP connect failed" -e "failed" -e "Interaction with CUPS failed" -c`
                        if [ $iot_status -gt "0" ]; then
                                iot_online=0
                                echo "offline" > /var/iot/status
                        else
                                iot_online=1
                                echo "online" > /var/iot/status
                        fi
                else
					process_status=0
                fi
}

check_iot()
{

	if [[ $IOT_TYPE = "lorawan" ]]; then
		check_fwd
		iot_service=draginofwd
	elif [[ $IOT_TYPE = "station" ]]; then
		check_station
		iot_service=draginostation
	elif [[ $IOT_TYPE = "bridge" ]]; then
		check_fwd
		iot_service=chirpstack-gateway-bridge
	fi
}

reload_iot()
{
	cur_reload_time=`date +%s`
	if [ "`uci get gateway.general.server_type`" = "lorawan" ] && [ `expr $cur_reload_time - $last_reload_time` -gt 80 ];then
		echo "$(date) Reload $iot_service" >> /var/iot/iot_state
		systemctl restart draginofwd && logger -i -t "iot_keep_alive" reload IoT
		last_reload_time=`date +%s`
	fi


}


###Check the lighttpd
check_lighttpd()
{
	checklighttpd=$(pgrep lighttpd)
		if [[ -n $checklighttpd ]]; then
		        lighttpdstatus=0
		else
		        lighttpdstatus=1
		fi
}

###check ttn-lw-stack
check_ttn()
{

	checkttn=$(pgrep ttn)
	if [[ -n $checkttn ]]; then
		ttnstatus=0
	else
		ttnstatus=1
	fi
}

### check node-red
check_nodered()
{
	checknodered=$(pgrep node-red)
	if [[ -n $checknodered ]]; then
		noderedstatus=0
	else
		noderedstatus=1
	fi
}


###statr monment
while [ 1 ]
do
	sleep 15
	check_internet
	if [[ $internetstatus = 1 ]]; then
		echo none > /sys/class/leds/dragino\:green\:stat/trigger && echo 0 > /sys/class/leds/dragino\:red\:sys/brightness  #clear let
		if [[ $internet_offline = 0 ]]; then
			check_iot
			if [[ $process_status = 0 ]]; then
				logger -i -t "iot_keep_alive" "$(date) $iot_service had dead, start restating $iot_service"
				systemctl restart $iot_service
			elif [[ $process_status = 1 ]]; then
				if [[ $iot_online -eq 1 ]]; then
					logger -i -t "iot_keep_alive" "$(date) $iot_service is running and iot is online."
					echo none > /sys/class/leds/dragino\:green\:stat/trigger && echo 1 > /sys/class/leds/dragino\:green\:stat/brightness
				else
					logger -i -t "iot_keep_alive" "$(date) $iot_service is running but iot is offline."
					echo timer > /sys/class/leds/dragino\:green\:stat/trigger  &&  echo 1 > /sys/class/leds/dragino\:green\:stat/brightness
					iot_offlinet=$(date +%s)
					if [[ $last_iotoffline_check = 0 ]] || [[ -z last_iotoffline_check ]] ; then
						last_iotoffline_check=$(date +%s)
					fi
					if [[ `expr $iot_offlinet - $last_iotoffline_check` -gt 120 ]];then
						reload_iot
						last_iotoffline_check=$(date +%s)
					fi
				fi
			fi
		else
			#reload iot
			check_iot
			internet_offline=0
			logger -i -t "iot_keep_alive" "$(date),Network restored to accessible ."
			echo timer > /sys/class/leds/dragino\:green\:stat/trigger  &&  echo 1 > /sys/class/leds/dragino\:green\:stat/brightness
			echo "$(date) Network back to access, Start reload the current server: $iot_service" >> /var/iot/iot_state
			reload_iot &&  logger -i -t "iot_keep_alive" Reload IoT
		fi
	elif [[ $internetstatus = 0 ]]; then
		if [[ $internet_offline = 0 ]]; then
			internet_offline=1
			echo none > /sys/class/leds/dragino\:green\:stat/trigger && echo 0 > /sys/class/leds/dragino\:green\:stat/brightness
			echo 1 > /sys/class/leds/dragino\:red\:sys/brightness 
			echo "$(date) Network is unreachable" >> /var/iot/iot_state
		fi
	fi

	##wlan blink

	check_wlan_status
	if [[  $sta_status = 0 ]]; then
		if [[ $ncm_status = "connected" ]]; then
			echo none > /sys/class/leds/dragino\:red\:wlan/trigger && echo 1 > /sys/class/leds/dragino\:red\:wlan/brightness
		else
			echo timer > /sys/class/leds/dragino\:red\:wlan/trigger && echo 1 > /sys/class/leds/dragino\:red\:wlan/brightness 
		fi
	else
		echo none > /sys/class/leds/dragino\:red\:wlan/trigger && echo 0 > /sys/class/leds/dragino\:red\:wlan/brightness 
	fi
done